"""
A standalone test runner script, since we don't ship a Django manage.py file.

"""

# SPDX-License-Identifier: BSD-3-Clause

from django.core.management import execute_from_command_line


def run_tests():
    """
    Run the test suite as Django's "manage.py test" would.

    """
    execute_from_command_line(["runtests.py", "test", "--verbosity", "2"])


if __name__ == "__main__":
    run_tests()



"""
Helper code for obtaining an Akismet API client.

"""

# SPDX-License-Identifier: BSD-3-Clause

import textwrap
import warnings

import akismet
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

_akismet_client = None  # pylint: disable=invalid-name


def _client_from_settings(client_class):
    """
    Attempt to obtain an Akismet client from legacy configuration in Django
    settings.

    """
    key = getattr(settings, "AKISMET_API_KEY", None)  # noqa: B009
    url = getattr(settings, "AKISMET_BLOG_URL", None)  # noqa: B009
    if not all([key, url]):
        return None
    warnings.warn(
        textwrap.dedent(
            """
            Specifying Akismet configuration via the Django settings AKISMET_API_KEY and
            AKISMET_BLOG_URL is deprecated and support for it will be removed in a
            future version of django-contact-form.

            Please migrate to specifying the configuration in the environment variables
            PYTHON_AKISMET_API_KEY and PYTHON_AKISMET_BLOG_URL. Or, if you cannot
            configure via environment variables, write a subclass of
            `AkismetContactForm` and override its `get_akismet_client()` method to
            construct your Akismet client.
            """
        ),
        DeprecationWarning,
        stacklevel=2,
    )
    client = client_class(config=akismet.Config(key, url))
    if client.verify_key(key, url):
        return client
    raise ImproperlyConfigured(
        "The Akismet configuration specified in your Django settings is invalid."
    )


def _client_from_environment(client_class):
    """
    Attempt to obtain an Akismet client from configuration in environment variables.

    """
    try:
        return client_class.validated_client()
    except akismet.ConfigurationError as exc:
        raise ImproperlyConfigured(
            "The Akismet configuration specified in your environment variables is "
            "missing or invalid."
        ) from exc


def _try_get_akismet_client(  # pylint: disable=inconsistent-return-statements
    client_class=akismet.SyncClient,
):
    """
    Attempt to obtain and return an instance of the given Akismet API client class..

    :raises django.core.exceptions.ImproperlyConfigured: When the Akismet client
       configuration is missing or invalid.

    """
    global _akismet_client  # pylint: disable=global-statement

    if _akismet_client is None:
        for attempt in [
            _client_from_settings,
            _client_from_environment,
        ]:
            _akismet_client = attempt(client_class)
            if _akismet_client is not None:
                return _akismet_client


def _clear_cached_instance():
    """
    Clear the cached Akismet API client instance, so that it will be re-created the
    next time it is requested.

    """
    global _akismet_client  # pylint: disable=global-statement
    _akismet_client = None



"""
Tests for the built-in views.

"""

# SPDX-License-Identifier: BSD-3-Clause

from http import HTTPStatus

import django
from django.conf import settings
from django.core import mail
from django.test import RequestFactory, TestCase
from django.test.utils import override_settings
from django.urls import reverse

from django_contact_form.forms import AkismetContactForm, ContactForm


@override_settings(ROOT_URLCONF="tests.test_urls")
class ContactFormViewTests(TestCase):
    """
    Tests for the built-in ContactFormView.

    """

    def test_get(self):
        """
        HTTP GET on the form view just shows the form.

        """
        contact_url = reverse("django_contact_form")

        response = self.client.get(contact_url)
        assert HTTPStatus.OK == response.status_code
        self.assertTemplateUsed(response, "django_contact_form/contact_form.html")

    def test_send(self):
        """
        Valid data through the view results in a successful send.

        """
        contact_url = reverse("django_contact_form")
        data = {"name": "Test", "email": "test@example.com", "body": "Test message"}

        response = self.client.post(contact_url, data=data)

        self.assertRedirects(response, reverse("django_contact_form_sent"))

        assert 1 == len(mail.outbox)

        message = mail.outbox[0]
        assert data["body"] in message.body
        assert settings.DEFAULT_FROM_EMAIL == message.from_email

        form = ContactForm(request=RequestFactory().request)
        assert form.recipient_list == message.recipients()

    def test_invalid(self):
        """
        Invalid data doesn't work.

        """
        contact_url = reverse("django_contact_form")
        data = {"name": "Test", "body": "Test message"}

        response = self.client.post(contact_url, data=data)

        assert HTTPStatus.OK == response.status_code
        # The argument signature of assertFormError() changed beginning in Django 4.1 --
        # prior to that the first argument was a response object, and after the
        # deprecation cycle completed in 5.0 the first argument is now the form
        # instance.
        if django.get_version() < "4.2":
            self.assertFormError(response, "form", "email", "This field is required.")
        else:
            self.assertFormError(
                response.context["form"], "email", "This field is required."
            )
        self.assertEqual(0, len(mail.outbox))

    def test_recipient_list(self):
        """
        Passing recipient_list when instantiating ContactFormView
        properly overrides the list of recipients.

        """
        contact_url = reverse("test_recipient_list")
        data = {"name": "Test", "email": "test@example.com", "body": "Test message"}

        response = self.client.post(contact_url, data=data)
        self.assertRedirects(response, reverse("django_contact_form_sent"))
        assert 1 == len(mail.outbox)

        message = mail.outbox[0]
        assert ["recipient_list@example.com"] == message.recipients()

    @override_settings(ROOT_URLCONF="django_contact_form.akismet_urls")
    def test_akismet_view(self):
        """
        The Akismet contact form URL uses a spam-filtering AkismetContactForm
        instance.

        """
        response = self.client.get(reverse("django_contact_form"))
        assert response.status_code == HTTPStatus.OK
        assert isinstance(response.context["form"], AkismetContactForm)



"""
Configuration file for the Sphinx documentation builder:

https://www.sphinx-doc.org/

"""

import os
import sys
from importlib.metadata import version as get_version

extensions = [
    "notfound.extension",
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
    "sphinx.ext.viewcode",
    "sphinxcontrib_django",
    "sphinxext.opengraph",
    "sphinx_copybutton",
    "sphinx_inline_tabs",
]
templates_path = ["_templates"]
source_suffix = ".rst"
master_doc = "index"
project = "django-contact-form"
copyright = "2007, James Bennett"
version = get_version("django-contact-form")
release = version
exclude_trees = ["_build"]
pygments_style = "sphinx"
htmlhelp_basename = "django-contact-formdoc"
html_theme = "furo"
latex_documents = [
    (
        "index",
        "django-contact-form.tex",
        "django-contact-form Documentation",
        "James Bennett",
        "manual",
    )
]

intersphinx_mapping = {
    "akismet": ("https://akismet.readthedocs.io/en/latest", None),
    "django": (
        "https://docs.djangoproject.com/en/stable/",
        "https://docs.djangoproject.com/en/stable/_objects/",
    ),
    "python": ("https://docs.python.org/3", None),
}

# Spelling check needs an additional module that is not installed by default.
# Add it only if spelling check is requested so docs can be generated without it.
if "spelling" in sys.argv:
    extensions.append("sphinxcontrib.spelling")

# Spelling language.
spelling_lang = "en_US"

# Location of word list.
spelling_word_list_filename = "spelling_wordlist.txt"

# OGP metadata configuration.
ogp_enable_meta_description = True
ogp_site_url = "https://django-contact-form.readthedocs.io/"

# Django settings for sphinxcontrib-django.
sys.path.insert(0, os.path.abspath("."))
django_settings = "docs_settings"



"""
Example URLConf for a contact form.

If all you want is the basic ContactForm with default behavior,
include this URLConf somewhere in your URL hierarchy (for example, at
``/contact/``)

"""

# SPDX-License-Identifier: BSD-3-Clause

from django.urls import path
from django.views.generic import TemplateView

from django_contact_form.views import ContactFormView

urlpatterns = [
    path("", ContactFormView.as_view(), name="django_contact_form"),
    path(
        "sent/",
        TemplateView.as_view(
            template_name="django_contact_form/contact_form_sent.html"
        ),
        name="django_contact_form_sent",
    ),
]



"""
Example URLConf for a contact form with Akismet spam filtering.

If all you want is the basic contact-form plus spam filtering,
include this URLConf somewhere in your URL hierarchy (for example, at
``/contact/``).

"""

# SPDX-License-Identifier: BSD-3-Clause

from django.urls import path
from django.views.generic import TemplateView

from .forms import AkismetContactForm
from .views import ContactFormView

urlpatterns = [
    path(
        "",
        ContactFormView.as_view(form_class=AkismetContactForm),
        name="django_contact_form",
    ),
    path(
        "sent/",
        TemplateView.as_view(
            template_name="django_contact_form/contact_form_sent.html"
        ),
        name="django_contact_form_sent",
    ),
]



"""
Tests for the Akismet spam-filtering integration.

"""

# SPDX-License-Identifier: BSD-3-Clause

import os
from http import HTTPStatus
from unittest import mock

import akismet
from django.core.exceptions import ImproperlyConfigured
from django.test import RequestFactory, TestCase
from django.test.utils import override_settings
from django.urls import reverse

from django_contact_form._akismet import _clear_cached_instance, _try_get_akismet_client
from django_contact_form.forms import AkismetContactForm


class AlwaysSpamClient(akismet.TestSyncClient):
    """
    Akismet client which always marks content as spam.

    """

    comment_check_response = akismet.CheckResponse.SPAM


class NeverSpamClient(akismet.TestSyncClient):
    """
    Akismet client which always marks content as non-spam.

    """

    comment_check_response = akismet.CheckResponse.HAM


class ValidConfigClient(akismet.TestSyncClient):
    """
    Akismet client which marks its configuration as valid.

    """

    verify_key_response = True


class InvalidConfigClient(akismet.TestSyncClient):
    """
    Akismet client which marks its configuration as invalid.

    """

    verify_key_response = False


@override_settings(ROOT_URLCONF="django_contact_form.akismet_urls")
class AkismetContactFormTests(TestCase):
    """
    Tests for the Akismet contact form.

    """

    akismet_config = akismet.Config(key="test-key", url="http://example.com")

    payload = {
        "name": "Test Name",
        "email": "test@example.com",
        "body": "Test message.",
    }

    def setUp(self):
        """
        Ensure the Akismet client instance is not cached between tests.

        """
        _clear_cached_instance()

    def request(self):
        """
        Construct and return an HttpRequest object for test use.

        """
        return RequestFactory().request()

    def test_akismet_form_spam(self):
        """
        The Akismet contact form correctly rejects spam.

        """
        with AlwaysSpamClient(config=self.akismet_config) as akismet_client, mock.patch(
            "django_contact_form._akismet._try_get_akismet_client",
            new=mock.Mock(return_value=akismet_client),
        ):
            form = AkismetContactForm(request=self.request(), data=self.payload)
            assert not form.is_valid()
            assert str(form.SPAM_MESSAGE) in form.errors["body"]

    def test_akismet_form_ham(self):
        """
        The Akismet contact form correctly accepts non-spam.

        """
        with NeverSpamClient(config=self.akismet_config) as akismet_client, mock.patch(
            "django_contact_form._akismet._try_get_akismet_client",
            new=mock.Mock(return_value=akismet_client),
        ):
            form = AkismetContactForm(request=self.request(), data=self.payload)
            assert form.is_valid()

    def test_akismet_form_no_body(self):
        """
        The Akismet contact form correctly skips validation when no email body is
        provided.

        """
        data = {"name": "Test", "email": "email@example.com"}

        with NeverSpamClient(config=self.akismet_config) as akismet_client, mock.patch(
            "django_contact_form._akismet._try_get_akismet_client",
            new=mock.Mock(return_value=akismet_client),
        ):
            form = AkismetContactForm(request=self.request(), data=data)
            assert not form.is_valid()

    def test_akismet_django_settings_valid(self):
        """
        When the Django settings are present and valid, an Akismet client is
        returned using them.

        """
        with self.settings(
            AKISMET_API_KEY=self.akismet_config.key,
            AKISMET_BLOG_URL=self.akismet_config.url,
        ):
            _try_get_akismet_client(ValidConfigClient)

    def test_akismet_django_settings_invalid(self):
        """
        When the Django settings are present and invalid, ImproperlyConfigured is
        raised.

        """
        with self.settings(
            AKISMET_API_KEY=self.akismet_config.key,
            AKISMET_BLOG_URL=self.akismet_config.url,
        ), self.assertRaises(ImproperlyConfigured):
            _try_get_akismet_client(InvalidConfigClient)

    def test_akismet_env_valid(self):
        """
        When the environment variables are present and valid, an Akismet client is
        returned using them.

        """
        try:
            os.environ["PYTHON_AKISMET_API_KEY"] = self.akismet_config.key
            os.environ["PYTHON_AKISMET_BLOG_URL"] = self.akismet_config.url
            _try_get_akismet_client(ValidConfigClient)
        finally:
            del os.environ["PYTHON_AKISMET_API_KEY"]
            del os.environ["PYTHON_AKISMET_BLOG_URL"]

    def test_akismet_env_invalid(self):
        """
        When the environment variables are present and invalid, ImproperlyConfigured
        is raised.

        """
        try:
            os.environ["PYTHON_AKISMET_API_KEY"] = self.akismet_config.key
            os.environ["PYTHON_AKISMET_BLOG_URL"] = self.akismet_config.url
            with self.assertRaises(ImproperlyConfigured):
                _try_get_akismet_client(InvalidConfigClient)
        finally:
            del os.environ["PYTHON_AKISMET_API_KEY"]
            del os.environ["PYTHON_AKISMET_BLOG_URL"]

    def test_akismet_env_missing(self):
        """
        When the environment variables are missing, ImproperlyConfigured is raised.

        """
        for key in ("PYTHON_AKISMET_API_KEY", "PYTHON_AKISMET_BLOG_URL"):
            if key in os.environ:
                del os.environ[key]
        with self.assertRaises(ImproperlyConfigured):
            _try_get_akismet_client(InvalidConfigClient)

    @override_settings(ROOT_URLCONF="django_contact_form.akismet_urls")
    def test_akismet_view(self):
        """
        The Akismet contact form URL uses a spam-filtering AkismetContactForm
        instance.

        """
        response = self.client.get(reverse("django_contact_form"))
        assert response.status_code == HTTPStatus.OK
        assert isinstance(response.context["form"], AkismetContactForm)



"""
Minimal Django settings file for test runs.

"""

# SPDX-License-Identifier: BSD-3-Clause

import pathlib

from django.utils.crypto import get_random_string

APP_DIR = pathlib.Path(__file__).parents[0]

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"
DEFAULT_FROM_EMAIL = "noreply@example.com"
DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"}}
INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "django_contact_form",
    "tests",
)
MANAGERS = [("Manager", "noreply@example.com")]
MIDDLEWARE = (
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
)
ROOT_URLCONF = "django_contact_form.urls"
SECRET_KEY = get_random_string(12)
SITE_ID = 1
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [APP_DIR / "templates"],
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]



"""
View which can render and send email from a contact form.

"""

# SPDX-License-Identifier: BSD-3-Clause

from django.urls import reverse_lazy
from django.views.generic.edit import FormView

from .forms import ContactForm


class ContactFormView(FormView):
    """
    The base view class from which most custom contact-form views should inherit. If
    you don't need any custom functionality, and are content with the default
    :class:`~django_contact_form.forms.ContactForm` class, you can also use it as-is
    (and the provided URLConf, ``django_contact_form.urls``, does exactly this).

    This is a subclass of Django's :class:`~django.views.generic.edit.FormView`, so
    refer to the Django documentation for a list of attributes/methods which can be
    overridden to customize behavior.

    One non-standard attribute is defined here:

    .. attribute:: recipient_list

       The list of email addresses to send mail to. If not specified, defaults to the
       :attr:`~django_contact_form.forms.ContactForm.recipient_list` of the form.

    Additionally, the following standard (from
    :class:`~django.views.generic.edit.FormView`) attributes are commonly useful to
    override (all attributes below can also be passed to
    :meth:`~django.views.generic.base.View.as_view()` in the URLconf, permitting
    customization without the need to write a full custom subclass of
    :class:`ContactFormView`). Each of these can be supplied as an attribute, or as a
    method with the name prefixed with ``get_`` (for example, a ``get_form_class()``
    method instead of a ``form_class`` attribute):

    .. attribute:: form_class

       The form class to use. By default, will be
       :class:`~django_contact_form.forms.ContactForm`.

    .. attribute:: template_name

       A :class:`str`, the template to use when rendering the form. By default, will be
       ``django_contact_form/contact_form.html``.

    .. attribute:: success_url

       The URL to redirect to after successful form submission. Can be a hard-coded
       string, the string resulting from calling Django's :func:`~django.urls.reverse`
       helper, or the lazy object produced by Django's :func:`~django.urls.reverse_lazy`
       helper. Default value is the result of calling :func:`~django.urls.reverse_lazy`
       with the URL name ``"django_contact_form_sent"``.

    You can also override the following method for full customization of the form
    instance construction:

    .. automethod:: get_form_kwargs

    """

    form_class = ContactForm
    recipient_list = None
    success_url = reverse_lazy("django_contact_form_sent")
    template_name = "django_contact_form/contact_form.html"

    def form_valid(self, form):
        """
        Handle a valid form by sending the email.

        """
        form.save()
        return super().form_valid(form)

    def get_form_kwargs(self) -> dict:
        """
        Return additional keyword arguments (as a dictionary) to pass to the form class
        on initialization.

        By default, this will return a dictionary containing the current
        :class:`~django.http.HttpRequest` (as the key ``request``) and, if
        :attr:`~ContactFormView.recipient_list` was defined, its value (as the key
        ``recipient_list``).

        .. warning:: **Request is a required argument**

           If you override :meth:`get_form_kwargs`, you **must** ensure that, at the very
           least, the keyword argument ``request`` is still provided, or
           :class:`~django_contact_form.forms.ContactForm` initialization will raise
           :exc:`TypeError`. The easiest approach is to use :class:`super` to call the
           base implementation in :class:`ContactFormView`, and modify the dictionary it
           returns.

        """
        # ContactForm instances require instantiation with an
        # HttpRequest.
        kwargs = super().get_form_kwargs()
        kwargs.update({"request": self.request})

        # We may also have been given a recipient list when
        # instantiated.
        if self.recipient_list is not None:
            kwargs.update({"recipient_list": self.recipient_list})
        return kwargs



"""
A base contact form for allowing users to send email messages through a web
interface.

"""

# SPDX-License-Identifier: BSD-3-Clause

from django import forms
from django.conf import settings
from django.contrib.sites.shortcuts import get_current_site
from django.core.mail import send_mail
from django.template import loader
from django.utils.translation import gettext_lazy as _


class ContactForm(forms.Form):
    """
    The base contact form class from which all contact form classes should inherit.

    If you don't need any customization, you can use this form to provide basic
    contact-form functionality; it will collect name, email address and message.

    The :class:`~django_contact_form.views.ContactFormView` included in this application
    knows how to work with this form and can handle many types of subclasses as well
    (see below for a discussion of the important points), so in many cases it will be
    all that you need. If you'd like to use this form or a subclass of it from one of
    your own views, here's how:

    1. When you instantiate the form, pass the current :class:`~django.http.HttpRequest`
       object as the keyword argument ``request``; this is used internally by the base
       implementation, and also made available so that subclasses can add functionality
       which relies on inspecting the request (such as spam filtering).

    2. To send the message, call the form's :meth:`save` method, which accepts the
       keyword argument ``fail_silently`` and defaults it to :data:`False`. This
       argument is passed directly to Django's :func:`~django.core.mail.send_mail`
       function, and allows you to suppress or raise exceptions as needed for
       debugging. The :meth:`save` method has no return value.

    Other than that, treat it like any other form; validity checks and validated data
    are handled normally, through the :meth:`~django.forms.Form.is_valid` method and the
    :attr:`~django.forms.Form.cleaned_data` dictionary.

    Under the hood, this form uses a somewhat abstracted interface in order to make it
    easier to subclass and add functionality.

    Customizing behavior in subclasses
    ``````````````````````````````````

    The following attributes play a role in determining behavior, and any of them can be
    implemented as an attribute or as a method (for example, if you wish to have
    :attr:`from_email` be dynamic, you can implement a method named :meth:`from_email`
    instead of setting the attribute :attr:`from_email`).

    .. attribute:: from_email

       The email address (:class:`str`) to use in the ``From:`` header of the
       message. By default, this is the value of the Django setting
       :setting:`DEFAULT_FROM_EMAIL`.

    .. attribute:: recipient_list

       A :class:`list` of recipients for the message. By default, this is the email
       addresses specified in the setting :setting:`MANAGERS`.

    .. attribute:: subject_template_name

       A :class:`str`, the name of the template to use when rendering the subject line
       of the message. By default, this is
       ``"django_contact_form/contact_form_subject.txt"``.

    .. attribute:: template_name

       A :class:`str`, the name of the template to use when rendering the body of the
       message. By default, this is ``"django_contact_form/contact_form.txt"``.

    And two methods are involved in producing the contents of the message to send:

    .. automethod:: message

    .. automethod:: subject

    Finally, the message itself is generated by the following two methods:

    .. automethod:: get_message_dict

    .. automethod:: get_message_context


    Other attributes/methods
    ````````````````````````

    Meanwhile, the following attributes/methods generally should not be overridden;
    doing so may interfere with functionality, may not accomplish what you want, and
    generally any desired customization can be accomplished in a more straightforward
    way through overriding one of the attributes/methods listed above.

    .. attribute:: request

       The :class:`~django.http.HttpRequest` object representing the current
       request. This is set automatically in ``__init__()``, and is used both to
       generate a :class:`~django.template.RequestContext` for the templates and to
       allow subclasses to engage in request-specific behavior.

    .. automethod:: save

    Note that subclasses which override ``__init__`` or :meth:`save` need to accept
    ``*args`` and ``**kwargs``, and pass them via :func:`super`, in order to preserve
    behavior (each of those methods accepts at least one additional argument, and this
    application expects and requires them to do so).

    """

    name = forms.CharField(max_length=100, label=_("Your name"))
    email = forms.EmailField(max_length=200, label=_("Your email address"))
    body = forms.CharField(widget=forms.Textarea, label=_("Your message"))

    from_email = settings.DEFAULT_FROM_EMAIL

    recipient_list = [mail_tuple[1] for mail_tuple in settings.MANAGERS]

    subject_template_name = "django_contact_form/contact_form_subject.txt"

    template_name = "django_contact_form/contact_form.txt"

    def __init__(
        self, *args, data=None, files=None, request=None, recipient_list=None, **kwargs
    ):
        if request is None:
            raise TypeError("Keyword argument 'request' must be supplied")
        self.request = request
        if recipient_list is not None:
            self.recipient_list = recipient_list
        super().__init__(data=data, files=files, *args, **kwargs)  # noqa: B026

    def message(self) -> str:
        """
        Return the body of the message to send. By default, this is accomplished by
        rendering the template name specified in :attr:`template_name`.

        """
        template_name = (
            self.template_name()  # pylint: disable=not-callable
            if callable(self.template_name)
            else self.template_name
        )
        return loader.render_to_string(
            template_name, self.get_message_context(), request=self.request
        )

    def subject(self) -> str:
        """
        Return the subject line of the message to send. By default, this is
        accomplished by rendering the template name specified in
        :attr:`subject_template_name`.

        .. warning:: **Subject must be a single line**

           The subject of an email is sent in a header (named ``Subject:``). Because
           email uses newlines as a separator between headers, newlines in the subject
           can cause it to be interpreted as multiple headers; this is the `header
           injection attack <https://en.wikipedia.org/wiki/Email_injection>`_. To
           prevent this, :meth:`subject` will always force the subject to a single line
           of text, stripping all newline characters. If you override :meth:`subject`,
           be sure to either do this manually, or use :class:`super` to call the parent
           implementation.

        """
        template_name = (
            self.subject_template_name()  # pylint: disable=not-callable
            if callable(self.subject_template_name)
            else self.subject_template_name
        )
        subject = loader.render_to_string(
            template_name, self.get_message_context(), request=self.request
        )
        return "".join(subject.splitlines())

    def get_message_context(self) -> dict:
        """
        Return the context used to render the templates for the email
        subject and body.

        The default context will be a :class:`~django.template.RequestContext` (using
        the current HTTP request, so user information is available), plus the contents
        of the form's :attr:`~django.forms.Form.cleaned_data` dictionary, and one
        additional variable:

        ``site``
          If ``django.contrib.sites`` is installed, the currently-active
          :class:`~django.contrib.sites.models.Site` object. Otherwise, a
          :class:`~django.contrib.sites.requests.RequestSite` object generated from the
          request.

        """
        if not self.is_valid():
            raise ValueError("Cannot generate Context from invalid contact form")
        return dict(self.cleaned_data, site=get_current_site(self.request))

    def get_message_dict(self) -> dict:
        """
        Generate the parts of the message and return them in a dictionary suitable
        for passing as keyword arguments to Django's
        :func:`~django.core.mail.send_mail`.

        By default, this method will collect and return :attr:`from_email`,
        :attr:`recipient_list`, :meth:`message` and :meth:`subject`. Overriding this
        allows essentially unlimited customization of how the message is generated. Note
        that for compatibility, implementations which override this should support
        callables for the values of :attr:`from_email` and :attr:`recipient_list`.

        """
        if not self.is_valid():
            raise ValueError("Message cannot be sent from invalid contact form")
        message_dict = {}
        for message_part in ("from_email", "message", "recipient_list", "subject"):
            attr = getattr(self, message_part)
            message_dict[message_part] = attr() if callable(attr) else attr
        return message_dict

    def save(self, fail_silently=False):
        """
        If the form has data and is valid, construct and send the email.

        By default, this is done by obtaining the parts of the email from
        :meth:`get_message_dict` and passing the result to Django's
        :func:`~django.core.mail.send_mail` function.

        """
        send_mail(fail_silently=fail_silently, **self.get_message_dict())


class AkismetContactForm(ContactForm):
    """
    A subclass of :class:`ContactForm` which adds spam filtering, via `the Akismet
    spam-detection service <https://akismet.com/>`_.

    Use of this class requires you to provide configuration for the Akismet web service.
    You'll need to obtain an Akismet API key, and you'll need to associate it with the
    site you'll use the contact form on. You can do this at <https://akismet.com/>. Once
    you have, put your Akismet API key in the environment variable
    ``PYTHON_AKISMET_API_KEY``, and the URL it's associated with in the environment
    variable ``PYTHON_AKISMET_BLOG_URL``.

    You will also need `the Python Akismet module <http://akismet.readthedocs.io/>`_ to
    communicate with the Akismet web service. You can install it manually (if you do,
    install at least version 24.5.0), or ``django-contact-form`` can install it
    automatically for you if you tell ``pip`` to install
    ``"django-contact-form[akismet]"``.

    Once you have an Akismet API key and URL configured, and the ``akismet`` module
    installed, you can drop in :class:`AkismetContactForm` anywhere you would have used
    :class:`ContactForm`. A URLconf is also provided in django-contact-form, at
    ``django_contact_form.akismet_urls``, which will set up :class:`AkismetContactForm`
    for you in place of the base contact form class.

    If you want to customize the spam-filtering behavior, there are two methods you can
    override:

    .. automethod:: get_akismet_check_arguments
    .. automethod:: get_akismet_client

    """

    SPAM_MESSAGE = _("Your message was classified as spam.")

    def get_akismet_check_arguments(self) -> dict:
        """
        Return the arguments which will be passed to the Akismet spam check.

        If your form contains additional fields which need to have their contents passed
        to Akismet, override this to ensure those arguments are correctly set.

        """
        return {
            "user_ip": self.request.META["REMOTE_ADDR"],
            "user_agent": self.request.META.get("HTTP_USER_AGENT"),
            "comment_author": self.cleaned_data.get("name"),
            "comment_author_email": self.cleaned_data.get("email"),
            "comment_content": self.cleaned_data["body"],
            "comment_type": "contact-form",
        }

    def get_akismet_client(self) -> "akismet.SyncClient":  # noqa: F821
        """
        Obtain and return an Akismet API client.

        By default, this will create a single API client instance and keep it resident
        in memory for the life of the Python process.

        If you need to customize the Akismet client creation (for example, to pass
        custom arguments to the Akismet API client), override this method.

        *Note:* Only synchronous Akismet clients (:class:`akismet.SyncClient`) are
        supported here; async clients (:class:`akismet.AsyncClient`) are not.

        """
        from ._akismet import (  # pylint: disable=import-outside-toplevel
            _try_get_akismet_client,
        )

        return _try_get_akismet_client()

    def clean_body(self):
        """
        Apply Akismet spam filtering to the submission.

        """
        akismet_client = self.get_akismet_client()

        if akismet_client.comment_check(**self.get_akismet_check_arguments()):
            raise forms.ValidationError(self.SPAM_MESSAGE)
        return self.cleaned_data["body"]



"""
Tests for the built-in form classes.

"""

# SPDX-License-Identifier: BSD-3-Clause

from django.conf import settings
from django.core import mail
from django.test import RequestFactory, TestCase

from django_contact_form.forms import ContactForm


class ContactFormTests(TestCase):
    """
    Tests for the base ContactForm.

    """

    valid_data = {"name": "Test", "email": "test@example.com", "body": "Test message"}

    def request(self):
        """
        Construct and return an HttpRequest object for test use.

        """
        return RequestFactory().request()

    def test_request_required(self):
        """
        Can't instantiate without an HttpRequest.

        """
        self.assertRaises(TypeError, ContactForm)

    def test_valid_data_required(self):
        """
        Can't try to build the message dict unless data is valid.

        """
        data = {"name": "Test", "body": "Test message"}
        form = ContactForm(request=self.request(), data=data)
        self.assertRaises(ValueError, form.get_message_dict)
        self.assertRaises(ValueError, form.get_message_context)

    def test_send(self):
        """
        Valid form can and does in fact send email.

        """
        form = ContactForm(request=self.request(), data=self.valid_data)
        assert form.is_valid()

        form.save()
        assert 1 == len(mail.outbox)

        message = mail.outbox[0]
        assert self.valid_data["body"] in message.body
        assert settings.DEFAULT_FROM_EMAIL == message.from_email
        assert form.recipient_list == message.recipients()

    def test_no_sites(self):
        """
        Sites integration works with or without installed
        contrib.sites.

        """
        with self.modify_settings(INSTALLED_APPS={"remove": ["django.contrib.sites"]}):
            form = ContactForm(request=self.request(), data=self.valid_data)
            assert form.is_valid()

            form.save()
            assert 1 == len(mail.outbox)

    def test_recipient_list(self):
        """
        Passing recipient_list when instantiating ContactForm properly
        overrides the list of recipients.

        """
        recipient_list = ["recipient_list@example.com"]
        form = ContactForm(
            request=self.request(), data=self.valid_data, recipient_list=recipient_list
        )
        assert form.is_valid()

        form.save()
        assert 1 == len(mail.outbox)

        message = mail.outbox[0]
        assert recipient_list == message.recipients()

    def test_callable_template_name(self):
        """
        When a template_name() method is defined, it is used and
        preferred over a 'template_name' attribute.

        """

        class CallableTemplateName(ContactForm):
            """
            Form with a template_name() method instead of a template_name attribute.

            """

            # pylint: disable=invalid-overridden-method
            def template_name(self):
                """
                Return the template name as a method rather than an attribute.

                """
                return "django_contact_form/test_callable_template_name.html"

        form = CallableTemplateName(request=self.request(), data=self.valid_data)
        assert form.is_valid()

        form.save()
        assert 1 == len(mail.outbox)

        message = mail.outbox[0]
        assert "Callable template_name used." in message.body

    def test_callable_message_parts(self):
        """
        Message parts implemented as methods are called and preferred
        over attributes.

        """
        overridden_data = {
            "from_email": "override@example.com",
            "message": "Overridden message.",
            "recipient_list": ["override_recpt@example.com"],
            "subject": "Overridden subject",
        }

        class CallableMessageParts(ContactForm):
            """
            Form with the message parts as methods rather than attributes.

            """

            def from_email(self):
                """
                Method returning the from_email.

                """
                return overridden_data["from_email"]

            def message(self):
                """
                Method returning the message.

                """
                return overridden_data["message"]

            def recipient_list(self):  # pylint: disable=method-hidden
                """
                Method returning the recipient_list.

                """
                return overridden_data["recipient_list"]

            def subject(self):
                """
                Method returning the subject.

                """
                return overridden_data["subject"]

        form = CallableMessageParts(request=self.request(), data=self.valid_data)
        assert form.is_valid()

        assert overridden_data == form.get_message_dict()



"""
URLConf for testing django-contact-form.

"""

# SPDX-License-Identifier: BSD-3-Clause

from django.urls import path
from django.views.generic import TemplateView

from django_contact_form.views import ContactFormView

urlpatterns = [
    path("", ContactFormView.as_view(), name="django_contact_form"),
    path(
        "sent/",
        TemplateView.as_view(
            template_name="django_contact_form/contact_form_sent.html"
        ),
        name="django_contact_form_sent",
    ),
    path(
        "test_recipient_list/",
        ContactFormView.as_view(recipient_list=["recipient_list@example.com"]),
        name="test_recipient_list",
    ),
]



"""
Automated testing via nox (https://nox.thea.codes/).

Combined with a working installation of nox (``pip install nox``), this file specifies a
matrix of tests, linters, and other quality checks which can be run individually or as a
suite.

To see available tasks, run ``nox --list``. To run all available tasks -- which requires
a functioning installation of at least one supported Python version -- run ``nox``. To
run a single task, use ``nox -s`` with the name of that task.

"""

import os
import pathlib
import shutil
import typing

import nox

nox.options.default_venv_backend = "venv"
nox.options.reuse_existing_virtualenvs = True

PACKAGE_NAME = "django_contact_form"

NOXFILE_PATH = pathlib.Path(__file__).parents[0]
ARTIFACT_PATHS = (
    NOXFILE_PATH / "src" / f"{PACKAGE_NAME}.egg-info",
    NOXFILE_PATH / "build",
    NOXFILE_PATH / "dist",
    NOXFILE_PATH / "__pycache__",
    NOXFILE_PATH / "src" / "__pycache__",
    NOXFILE_PATH / "src" / PACKAGE_NAME / "__pycache__",
    NOXFILE_PATH / "tests" / "__pycache__",
)


def clean(paths: typing.Iterable[os.PathLike] = ARTIFACT_PATHS) -> None:
    """
    Clean up after a test run.

    """
    [
        shutil.rmtree(path) if path.is_dir() else path.unlink()
        for path in paths
        if path.exists()
    ]


# Tasks which run the package's test suites.
# -----------------------------------------------------------------------------------


@nox.session(tags=["tests"])
@nox.parametrize(
    "python,django",
    [
        # Python/Django testing matrix. Tests Django 4.2, and 5.0, on Python 3.8
        # through 3.11, skipping unsupported combinations.
        (python, django)
        for python in ["3.8", "3.9", "3.10", "3.11", "3.12"]
        for django in ["4.2", "5.0"]
        if (python, django) not in [("3.8", "5.0"), ("3.9", "5.0")]
    ],
)
def tests_with_coverage(session: nox.Session, django: str) -> None:
    """
    Run the package's unit tests, with coverage report.

    """
    session.install(f"Django~={django}.0", ".[akismet,tests]")
    python_version = session.run(
        f"{session.bin}/python{session.python}", "--version", silent=True
    ).strip()
    django_version = session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "django",
        "--version",
        silent=True,
    ).strip()
    session.log(f"Running tests with {python_version}/Django {django_version}")
    session.run(f"{session.bin}/python{session.python}", "-Im", "coverage", "--version")
    session.run(
        f"{session.bin}/python{session.python}",
        "-Wmodule::DeprecationWarning",
        "-Im",
        "coverage",
        "run",
        "--source",
        PACKAGE_NAME,
        "runtests.py",
        env={"DJANGO_SETTINGS_MODULE": "tests.settings"},
    )
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "coverage",
        "report",
        "--show-missing",
    )
    clean()


# Tasks which test the package's documentation.
# -----------------------------------------------------------------------------------


@nox.session(python=["3.12"], tags=["docs"])
def docs_build(session: nox.Session) -> None:
    """
    Build the package's documentation as HTML.

    """
    session.install(".[docs]")
    session.chdir("docs")
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "sphinx",
        "-b",
        "html",
        "-d",
        f"{session.bin}/../tmp/doctrees",
        ".",
        f"{session.bin}/../tmp/html",
    )
    clean()


@nox.session(python=["3.12"], tags=["docs"])
def docs_docstrings(session: nox.Session) -> None:
    """
    Enforce the presence of docstrings on all modules, classes, functions, and
    methods.

    """
    session.install("interrogate")
    session.run(
        f"{session.bin}/python{session.python}", "-Im", "interrogate", "--version"
    )
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "interrogate",
        "-v",
        "src/",
        "tests/",
        "noxfile.py",
    )
    clean()


@nox.session(python=["3.12"], tags=["docs"])
def docs_spellcheck(session: nox.Session) -> None:
    """
    Spell-check the package's documentation.

    """
    session.install("pyenchant", "sphinxcontrib-spelling", ".[docs]")
    build_dir = session.create_tmp()
    session.chdir("docs")
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "sphinx",
        "-W",  # Promote warnings to errors, so that misspelled words fail the build.
        "-b",
        "spelling",
        "-d",
        f"{build_dir}/doctrees",
        ".",
        f"{build_dir}/html",
        # On Apple Silicon Macs, this environment variable needs to be set so
        # pyenchant can find the "enchant" C library. See
        # https://github.com/pyenchant/pyenchant/issues/265#issuecomment-1126415843
        env={"PYENCHANT_LIBRARY_PATH": os.getenv("PYENCHANT_LIBRARY_PATH", "")},
    )
    clean()


# Code formatting checks.
#
# These checks do *not* reformat code -- that happens in pre-commit hooks -- but will
# fail a CI build if they find any code that needs reformatting.
# -----------------------------------------------------------------------------------


@nox.session(python=["3.12"], tags=["formatters"])
def format_black(session: nox.Session) -> None:
    """
    Check code formatting with Black.

    """
    session.install("black")
    session.run(f"{session.bin}/python{session.python}", "-Im", "black", "--version")
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "black",
        "--check",
        "--diff",
        "src/",
        "tests/",
        "docs/",
        "noxfile.py",
    )
    clean()


@nox.session(python=["3.12"], tags=["formatters"])
def format_isort(session: nox.Session) -> None:
    """
    Check code formating with Black.

    """
    session.install("isort")
    session.run(f"{session.bin}/python{session.python}", "-Im", "isort", "--version")
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "isort",
        "--check-only",
        "--diff",
        "src/",
        "tests/",
        "docs/",
        "noxfile.py",
    )
    clean()


# Linters.
# -----------------------------------------------------------------------------------


@nox.session(python=["3.12"], tags=["linters", "security"])
def lint_bandit(session: nox.Session) -> None:
    """
    Lint code with the Bandit security analyzer.

    """
    session.install("bandit[toml]")
    session.run(f"{session.bin}/python{session.python}", "-Im", "bandit", "--version")
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "bandit",
        "-c",
        "./pyproject.toml",
        "-r",
        "src/",
        "tests/",
    )
    clean()


@nox.session(python=["3.12"], tags=["linters"])
def lint_flake8(session: nox.Session) -> None:
    """
    Lint code with flake8.

    """
    session.install("flake8", "flake8-bugbear")
    session.run(f"{session.bin}/python{session.python}", "-Im", "flake8", "--version")
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "flake8",
        "src/",
        "tests/",
        "docs/",
        "noxfile.py",
    )
    clean()


@nox.session(python=["3.12"], tags=["linters"])
def lint_pylint(session: nox.Session) -> None:
    """
    Lint code with Pyling.

    """
    # Pylint requires that all dependencies be importable during the run, so unlike
    # other lint tasks we just install the package.
    session.install(".[akismet]")
    session.install("pylint", "pylint-django")
    session.run(f"python{session.python}", "-Im", "pylint", "--version")
    session.run(f"python{session.python}", "-Im", "pylint", "src/", "tests/")


# Packaging checks.
# -----------------------------------------------------------------------------------


@nox.session(python=["3.12"], tags=["packaging"])
def package_build(session: nox.Session) -> None:
    """
    Check that the package builds.

    """
    clean()
    session.install("build")
    session.run(f"{session.bin}/python{session.python}", "-Im", "build", "--version")
    session.run(f"{session.bin}/python{session.python}", "-Im", "build")


@nox.session(python=["3.12"], tags=["packaging"])
def package_description(session: nox.Session) -> None:
    """
    Check that the package description will render on the Python Package Index.

    """
    package_dir = session.create_tmp()
    session.install("build", "twine")
    session.run(f"{session.bin}/python{session.python}", "-Im", "build", "--version")
    session.run(f"{session.bin}/python{session.python}", "-Im", "twine", "--version")
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "build",
        "--wheel",
        "--outdir",
        f"{package_dir}/build",
    )
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "twine",
        "check",
        f"{package_dir}/build/*",
    )
    clean()


@nox.session(python=["3.12"], tags=["packaging"])
def package_manifest(session: nox.Session) -> None:
    """
    Check that the set of files in the package matches the set under version control.

    """
    session.install("check-manifest")
    session.run(
        f"{session.bin}/python{session.python}", "-Im", "check_manifest", "--version"
    )
    session.run(
        f"{session.bin}/python{session.python}", "-Im", "check_manifest", "--verbose"
    )
    clean()


@nox.session(python=["3.12"], tags=["packaging"])
def package_pyroma(session: nox.Session) -> None:
    """
    Check package quality with pyroma.

    """
    session.install("pyroma")
    session.run(
        f"{session.bin}/python{session.python}",
        "-c",
        'from importlib.metadata import version; print(version("pyroma"))',
    )
    session.run(f"{session.bin}/python{session.python}", "-Im", "pyroma", ".")
    clean()


@nox.session(python=["3.12"], tags=["packaging"])
def package_wheel(session: nox.Session) -> None:
    """
    Check the built wheel package for common errors.

    """
    package_dir = session.create_tmp()
    session.install("build", "check-wheel-contents")
    session.run(f"{session.bin}/python{session.python}", "-Im", "build", "--version")
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "check_wheel_contents",
        "--version",
    )
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "build",
        "--wheel",
        "--outdir",
        f"{package_dir}/build",
    )
    session.run(
        f"{session.bin}/python{session.python}",
        "-Im",
        "check_wheel_contents",
        f"{package_dir}/build",
    )
    clean()
